#include "lowqafile.h"
#include "ptools.h"
#include "TFile.h"
#include "TKey.h"

#include <iostream>
using namespace std;

LowQAFile::LowQAFile(const QString &fName)
{
    fileName = new QString(fName);
    nHistograms = 0;
    status.setAutoDelete(TRUE);
    status.clear();
    scanHists();
    status.resize(calcNearestPrime(nHistograms));
}

LowQAFile::~LowQAFile(void)
{
    status.clear();
}

TH1 *LowQAFile::getQAHistogram(QString *name, QString *path, bool check)
{
    TFile *f;
    TKey *key;
    TH1 *h;
    
    if(check && (getStatus((char *)name->latin1()) != BAD))
    {
	return NULL;
    }
    else
    {
	f = new TFile(fileName->latin1(),"READ");
	if(f->IsOpen())
	{
	    f->cd(path->latin1());
	    key = f->FindKeyAny(name->latin1());
	    if(key)
	    {
		h = (TH1 *)key->ReadObj()->Clone();
		h->SetDirectory(0);
		delete f;
		return h;
	    }
	    else
	    {
		delete f;
		return NULL;
	    }
	}
	else
	{
	    delete f;
	    return NULL;
	}
    }
}


void LowQAFile::getMeanValue(QString *name, QString *path, float *mean, float *err)
{
   TH1 *h;
   
   h = getQAHistogram(name,path);
   if(h)
   {
       *mean = h->GetMean();
       *err = h->GetRMS();
   }
   else
   {
       *mean = 0;
       *err = 0;
   }
   delete h;
}

void LowQAFile::getMeanValueTrend(QString *name, QString *path, float *mean, float *err)
{
    TH1 *h;
    TH1 *p;
    
    h = getQAHistogram(name,path);
    if(h)
    {
	p = PTools::getBinContentDistribution(h);
	*mean = p->GetMean();
	*err = p->GetRMS();
	delete p;
    }
    else
    {
	*mean = 0;
	*err = 0;
    }
    delete h;
}


void LowQAFile::scanHists(void)
{
    TFile *file;
    
    file = new TFile(fileName->latin1(),"READ");
    if(file->IsOpen())
    {
	file->cd();
	scanFile(&nHistograms);
   }
    delete file;
}

void LowQAFile::scanFile(int *n)
{
    TList *dir=0;
    TIterator *iter=0;
    TKey *key;
    
   dir = gDirectory->GetListOfKeys();
    if(dir)
    {
	iter = dir->MakeIterator();
	while((key = (TKey *)iter->Next()) != NULL)
	{
	    if(key->IsFolder())
	    {
		gDirectory->cd(key->GetName());
		scanFile(n);
	    }
	    else
	    {
		(*n)++;
	    }
	}
	delete iter;
	gDirectory->cd("..");
    }
}


int LowQAFile::calcNearestPrime(int n)
{
    int ending;
    int remainder=1;
    int prime=0;
    
    ending = n+1000;
    
    for(int count=n;count<=ending;count++)
    {
	remainder = 1;
	for(int divisor = 2;((divisor < count) && (remainder!=0));divisor++)
	{
	    remainder = count % divisor;
	    if ((remainder!=0) && (divisor==count - 2))
	    {
		prime++;
		return count;
	    }
	}
    }
    return 0;
}	

int LowQAFile::getStatus(const QString name)
{
  if(status[name.latin1()])
    {
	//cout << *(status.find(name.latin1())) << endl;
	return *(status[name.latin1()]);
    }
    else
	return 0;
}

bool LowQAFile::updateHistogramStatus(const QString name, int stat, bool force)
{
    int *n;
    
    n = new int;
    *n = stat; 
     if(status[name.latin1()])
    {
	if((getStatus(name.latin1()) < stat) || force)
	{
	    status.replace(name.latin1(),n);
	    return TRUE;
	}
	else
	{
	    delete n;
	    return FALSE;
	}
    }
    else
    {
	status.insert(name.latin1(),n);
	return TRUE;
    }
    return FALSE;
}


void LowQAFile::print(void)
{
    QAsciiDictIterator<int> it(status);
    for(;it.current();++it)
    {
	 cout << it.currentKey() << ": " << *(it.current()) << endl;
    }
}

QDomNode LowQAFile::getStatusXml(QDomDocument *doc)
{
    QAsciiDictIterator<int> it(status);
    int i=0;
    
    hName = new QDomElement[nHistograms];
    fName = doc->createElement("file");
    fName.setAttribute("name",*fileName);
    for(;it.current();++it)
    {
	hName[i] = doc->createElement("histogram");
	hName[i].setAttribute("name", QString(it.currentKey()));
	hName[i].setAttribute("status",*(it.current()));
	fName.appendChild(hName[i]);
	i++;
    }
    return fName;
 
}

void LowQAFile::readHistStatus(QDomNode &node)
{
    QDomElement e1;
    QDomNode n;
    QString hName;
    int hStatus;
    
    n = node.firstChild();
    while(!n.isNull())
    {
	e1 = n.toElement();
	if(e1.tagName().localeAwareCompare("histogram") == 0)
	{
	    hName = e1.attribute("name");
	    hStatus = e1.attribute("status").toInt();
	    updateHistogramStatus(hName,hStatus,TRUE);
	}
	n = e1.nextSibling();
    }
    
}
